%
%    This is the driver routine for the case in which d=3 of software
%    package for computing the elastic registration of two simple curves
%    in d-dimensional space and the elastic shape distance between them.
%
%*******************************************************************
%
%    Author: Javier Bernal
%            National Institute of Standards and Technology (NIST)
%
%     Disclaimer:
%
%     This software was developed at the National Institute of Standards
%     and Technology by employees of the Federal Government in the
%     course of their official duties. Pursuant to title 17 Section 105
%     of the United States Code this software is not subject to
%     copyright protection and is in the public domain. This software is
%     experimental. NIST assumes no responsibility whatsoever for its
%     use by other parties, and makes no guarantees, expressed or
%     implied, about its quality, reliability, or any other
%     characteristic. We would appreciate acknowledgement if the
%     software is used.
%
clear;
%
%tic;
%fprintf('\n \n');
%
% Driver routine for computing elastice shape distance between two
% curves in 2-d
%
stage = 3;
% stage = integer for determining level of methodology used
fprintf('\n Level of methodology is %2.0f \n', stage);
if stage == 3
   fprintf('\n If at least one curve is not closed it may change to 2 \n');
end
%fprintf('\n \n');
iten = 7;
% iten = maximum number of while loop iterations allowed in the main procedure
%fprintf('\n \n');
%
space = 1;
% space = spacing plus 1 between starting points on closed curve if any
fprintf('\n If at least one curve is closed, spacing plus 1 between starting points is %2.0f \n', space);
%fprintf('\n \n');
itop = 1;
% itop = constant number of times Dynamic Programming algorithm is executed if at least one curve is closed
fprintf('\n If at least one curve is closed, # of times per iteration DP algo is executed = %2.0f \n', itop);
%fprintf('\n \n');
%
dmn_t_is_read = 1;
% = 1 if domains read for each curve (not necessarily the same)
% = 0 if domains computed uniformly for each curve (but not
%     necessarily the same)
if (dmn_t_is_read)
   fprintf('\n Domains read for each curve \n');
%   fprintf('\n \n');
else
   fprintf('\n Domains computed for each curve \n');
%   fprintf('\n \n');
end
%
both_drct = 0;
% both_drct is for determining whether the first curve is to be
% considered in the reversed direction (0 if no, 1 if yes) as well
if (both_drct)
   fprintf('\n First curve considered in both directions \n');
%   fprintf('\n \n');
else
   fprintf('\n First curve considered in given direction only \n');
%   fprintf('\n \n');
end
%
% clsd_crve = number of curves that are closed (0, 1 or 2)
% whch_crve = if only one curve is closed, which curve it is (1 or 2)
% drct_crve = curve 1 direction for optimality (1 [as given] -1 [counter])
% strt_crve = optimal starting point minus 1 on first closed curve if any
% R = optimal rotation

if(dmn_t_is_read)

   f1 = fopen('ellip1.txt');
%  f1 = fopen('helix1.txt');
   p1 = fscanf(f1, '%g %g %g %g', [4 inf]);
   fclose(f1);
%
   f2 = fopen('ellip2.txt');
%  f2 = fopen('helix2.txt');
   p2 = fscanf(f2, '%g %g %g %g', [4 inf]);
   fclose(f2);
%
   N1 = length(p1);
   N2 = length(p2);
%
   for i = 1:N1
       r1(i) = p1(1,i);
       b1(1,i) = p1(2,i);
       b1(2,i) = p1(3,i);
       b1(3,i) = p1(4,i);
   end
   for i = 1:N2
       r2(i) = p2(1,i);
       b2(1,i) = p2(2,i);
       b2(2,i) = p2(3,i);
       b2(3,i) = p2(4,i);
   end
%min(r1)
%max(r1)
%min(r2)
%max(r2)
%
   dt = max(r1)-min(r1);
   s1 = (r1-min(r1))/dt;
   s1(1) = 0;
   s1(N1) = 1;
%
   dt = max(r2)-min(r2);
   s2 = (r2-min(r2))/dt;
   s2(1) = 0;
   s2(N2) = 1;
%
%f1=b1;
%f2=b2;
%
%imath=0;
%if(imath)
%   ispan=25
%   tic;
%   f1 = smooth(s1,b1,ispan);
%   f2 = smooth(s2,b2,ispan);
%   f1=f1';
%   f2=f2';
%   timing_smooth = toc;
%   timing_smooth
%end
%
   [f1,f2,t1,t2,gammabest,iter,Ecc,Ec,minE,clsd_crve,whch_crve,drct_crve,strt_crve,R] = ESD_comp_alldim(stage,iten,space,itop,both_drct,b1,b2,s1,s2);
%
else
   f1 = fopen('ellip1_0.txt');
%  f1 = fopen('helix1_0.txt');
   b1 = fscanf(f1, '%g %g %g', [3 inf]);
   fclose(f1);
%
   f2 = fopen('ellip2_0.txt');
%  f2 = fopen('helix2_0.txt');
   b2 = fscanf(f2, '%g %g %g', [3 inf]);
   fclose(f2);
%
   [f1,f2,t1,t2,gammabest,iter,Ecc,Ec,minE,clsd_crve,whch_crve,drct_crve,strt_crve,R] = ESD_comp_alldim(stage,iten,space,itop,both_drct,b1,b2);
%
   N1 = length(b1);
   N2 = length(b2);
   s1 = linspace(0,1,N1);
   s2 = linspace(0,1,N2);
end
%
%timing = toc;

[n1,N1]=size(b1);
[n2,N2]=size(b2);
[m1,M1]=size(f1);
[m2,M2]=size(f2);

   if(clsd_crve == 1 && whch_crve == 2)
      t = t2;
      if(strt_crve == 0)
         f2n = f2;
      else
         f2n = ShiftF(f2,strt_crve);
      end
      if n1 ~= 1
         f2best = R*f2n;
%        f1best = Group_Action_by_Gamma_Coord(f1,gammabest);
         f1best = spline(t1,f1,gammabest);
      else
         f2best = f2n;
         f1best = spline(t1,f1,gammabest);
      end
   else
      t = t1;
      if(strt_crve == 0)
         f1n = f1;
      else
         f1n = ShiftF(f1,strt_crve);
      end
      if n1 ~= 1
         f1best = R*f1n;
%        f2best = Group_Action_by_Gamma_Coord(f2,gammabest);
         f2best = spline(t2,f2,gammabest);
      else
         f1best = f1n;
         f2best = spline(t2,f2,gammabest);
      end
   end

%figure;
%plot(b1(1,:),b1(2,:),'b-');
%figure;
%plot(b2(1,:),b2(2,:),'r-');
%figure;
%plot(f1best(1,:),f1best(2,:),'b-');
%figure;
%plot(f2best(1,:),f2best(2,:),'r-');

p3(1,:) = t;
p3(2,:) = gammabest;
f1 = fopen('gamma_data.txt', 'w');
fprintf(f1,'%12.8f %12.8f \n',p3);
fclose(f1);

f3 = fopen('f1best_data.txt', 'w');
fprintf(f3,'%12.8f %12.8f %12.8f \n',f1best);
fclose(f3);

f3 = fopen('f2best_data.txt', 'w');
fprintf(f3,'%12.8f %12.8f %12.8f \n',f2best);
fclose(f3);

%figure;
%plot(t,gammabest,'b-');
%
fprintf('\n Dimension of curves is %5.0f \n', n1);
%fprintf('\n \n');
fprintf('\n Numbers of points on curves on input == %5.0f %5.0f \n', N1, N2);
fprintf('\n Numbers of points on curves on output = %5.0f %5.0f \n', M1, M2);
%fprintf('\n \n');
fprintf('\n Number of closed curves (0, 1 or 2) = %5.0f \n', clsd_crve);
%fprintf('\n \n');
fprintf('\n ********************* Note in general: \n');
fprintf('\n If at least one curve is closed, domains of curves are the same uniform partition \n');
fprintf('\n unless the dimension of the curves is 1 and level of methodology used is at stage 1 \n');
fprintf('\n in which case domains do not have to be equal \n');
%fprintf('\n \n');
fprintf('\n If neither curve is closed, domains of curves are equal but not necessarily uniform \n');
fprintf('\n unless the dimension of the curves is 1 in which case domains do not have to be equal \n');
fprintf('\n ********************* \n');
%fprintf('\n \n');
fprintf('\n For the current problem: \n');
if(clsd_crve == 1 && whch_crve == 2)
   fprintf('\n Only the second curve is closed thus any point in it can be a starting point for it \n');
   fprintf('\n This curve may be rotated for each of its starting points if dimension of curves > 1 \n');
   fprintf('\n Note that if dimension of curves is 1, the concept of rotations is not used \n');
   fprintf('\n However the concept of a maximal trace may be used \n');
   fprintf('\n Optimal starting point minus 1 identified on this curve = %5.0f \n', strt_crve);
%   fprintf('\n \n');
elseif clsd_crve == 0
   fprintf('\n Neither curve is closed \n');
   fprintf('\n If dimension of curves > 1, first curve may be rotated for its only starting point \n');
   fprintf('\n Note that if dimension of curves is 1, the concept of rotations is not used \n');
%   fprintf('\n \n');
else
   fprintf('\n The first curve is closed thus any point in it can be a starting point for it \n');
   fprintf('\n This curve may be rotated for each of its starting points if dimension of curves > 1 \n');
   fprintf('\n Note that if dimension of curves is 1, the concept of rotations is not used \n');
   fprintf('\n However the concept of a maximal trace may be used \n');
   fprintf('\n Optimal starting point minus 1 identified on this curve = %5.0f \n', strt_crve);
%   fprintf('\n \n');
end
if (drct_crve == 1)
   fprintf('\n Direction of first curve for optimality is as given \n');
%   fprintf('\n \n');
else
   fprintf('\n Direction of first curve for optimality is reversed \n');
%   fprintf('\n \n');
end
%fprintf('\n \n');
%fprintf('\n Execution time = %5.5f \n', timing);
%fprintf('\n \n');
if clsd_crve ~= 2 && stage == 3
   stage = 2;
end
fprintf('\n Actual level of methodology is %2.0f \n', stage);
%fprintf('\n \n');
if stage == 2 || stage == 3
   fprintf('\n Number of while loop iterations and maximum number allowed = %2.0f %2.0f \n',iter,iten);
   fprintf('\n Values of last two distances at the end of while loop = %9.5f %9.5f \n',Ecc,Ec);
end
%fprintf('\n \n');
fprintf('\n Computed distance = %5.9f \n', minE);
%fprintf('\n \n');
fprintf('\n Computed optimal matrix = \n');
fprintf('\n %5.5f %5.5f %5.5f \n', R(1,1), R(1,2), R(1,3));
fprintf('\n %5.5f %5.5f %5.5f \n', R(2,1), R(2,2), R(2,3));
fprintf('\n %5.5f %5.5f %5.5f \n', R(3,1), R(3,2), R(3,3));
fprintf('\n \n');
%
