function result = L2_inner_prod(f1,f2,varargin)
%L2_INNER_PROD computes the discrete L2 inner product of two arrays.
%
%   SUM = L2_INNER_PROD(B1,B2) uses the trapezoidal rule to compute 
%   a discrete approximation to the L2 inner product of two vector-valued 
%   functions b1(t), b2(t), with t in [0,1]. The two functions are provided 
%   as discrete samples with N equi-spaced points. Thus the two samples 
%   B1, B2 should be arrays of the same size with dimensions DIM x N.
%
%   ERROR = L2_INNER_PROD(F1,F2,T) optionally a nonuniform discretization
%   of the domain T can be passed to the function if the function values
%   are obtained on a nonuniform grid: T(1)=0 < T(2) < ... < T(N)=1.

N = size(f1,2);
if size(f2,2) ~= N
    error('The number of columns in f1,f2 has to be equal!')
end

if isempty(varargin)
    t = 0;
else 
    t = varargin{1};
    if length(t) ~= N
        error('The length of t has to be the same as the number of columns in q!');
    end
end

if isscalar(t) % Uniformly discretized with const spacing h.
    h = 1/(N-1);
    result = h * trapz(sum(f1.*f2,1));
else % Nonuniformly discretized at grid points t.
    result = trapz( t, sum(f1.*f2,1) );
end
