function [b1,b2,t] = parameterize_curve_pair(b1_in,b2_in,varargin)
%
% [B1_NEW,B2_NEW,T] = PARAMETERIZE_CURVE_PAIR( B1, B2, T1, T2, DT_TOL,
%                                              INTERPOLATION_METHOD )

dt_tol = 0.00001;
%interpolation_method = 'linear';
interpolation_method = 'cubic';

if isempty(varargin)
    t1 = arclen_fct_values(b1_in);
    t2 = arclen_fct_values(b2_in);
else
    t1 = varargin{1};
    if length(varargin) < 2
        t2 = arclen_fct_values(b2_in);
    else
        t2 = varargin{2};
        if length(varargin) > 2
            dt_tol = varargin{3};
            if length(varargin) > 3
                interpolation_method = varargin{4};
            end
        end
    end
end

t = union(t1,t2);
N = length(t);

remove = false(1,N);  % Initialize all remove flags to false.
remove(2:N) = (t(2:N) - t(1:N-1)) < dt_tol;
t(remove) = [];

if strcmp(interpolation_method,'cubic')
    b1 = spline(t1, b1_in, t);
    b2 = spline(t2, b2_in, t);
else % interpolation_method = 'linear' or something else
    dim = size(b1_in,1);
    N = length(t);
    b1 = zeros(dim,N);
    b2 = zeros(dim,N);
    for k=1:dim
        b1(k,:) = interp1(t1, b1_in(k,:), t, interpolation_method);
        b2(k,:) = interp1(t2, b2_in(k,:), t, interpolation_method);
    end
end
